# -*- coding: utf8 -*-
from django.utils import unittest
from basis.utils import get_user_model
from .models import Person


class TestLoginCodes(unittest.TestCase):

    def setUp(self):
        self.user1 = get_user_model().objects.get_or_create(username="test1")[0]
        self.user2 = get_user_model().objects.get_or_create(username="test2")[0]

    def tearDown(self):
        self.user1.delete()
        self.user2.delete()

        Person.all_objects.all().delete()

    def test_crate_person_without_user(self):
        person = Person()
        person.save()

        self.assertEqual(person.created_by, None)
        self.assertEqual(person.updated_by, None)

    def test_create_person_with_user(self):

        person = Person()
        person.save(current_user=self.user1)

        self.assertEqual(person.created_by, self.user1)
        self.assertEqual(person.updated_by, self.user1)

    def test_update_person_with_user(self):

        person = Person()
        person.save(current_user=self.user1)

        self.assertEqual(person.created_by, self.user1)
        self.assertEqual(person.updated_by, self.user1)

        person.save(current_user=self.user2)

        self.assertEqual(person.created_by, self.user1)
        self.assertEqual(person.updated_by, self.user2)

    def test_delete_person(self):

        person = Person()
        person.save(current_user=self.user1)

        self.assertEqual(Person.objects.all().count(), 1)
        person.delete()
        self.assertEqual(Person.objects.all().count(), 0)
        self.assertEqual(Person.all_objects.all().count(), 1)

        person.restore()
        self.assertEqual(Person.objects.all().count(), 1)



# -*- coding: utf-8 -*-
import django
from django.conf import settings

if django.VERSION >= (1, 5):
    AUTH_USER_MODEL = settings.AUTH_USER_MODEL
else:
    AUTH_USER_MODEL = 'auth.User'


def get_user_model():
    if django.VERSION >= (1, 5):
        from django.contrib.auth import get_user_model
        return get_user_model()
    else:
        from django.contrib.auth.models import User
        return User


def get_username(user):
    try:
        return user.get_username()
    except AttributeError:
        return user.username



from datetime import datetime

from django.db import models

from .utils import AUTH_USER_MODEL
from .managers import BasisModelManager


class BasisModel(models.Model):
    deleted = models.BooleanField(default=False, editable=False)

    created_at = models.DateTimeField(default=datetime.now, editable=False, auto_now_add=True)
    created_by = models.ForeignKey(AUTH_USER_MODEL, null=True, default=None, editable=False, related_name="%(class)s_created")

    updated_at = models.DateTimeField(default=datetime.now, editable=False, auto_now=True)
    updated_by = models.ForeignKey(AUTH_USER_MODEL, null=True, default=None, editable=False, related_name="%(class)s_updated")

    objects = BasisModelManager()
    all_objects = models.Manager()

    class Meta:
        abstract = True

    def save(self, *args, **kwargs):

        if 'current_user' in kwargs:
            current_user = kwargs.pop("current_user")

            if not self.id:
                self.created_by = current_user

            self.updated_by = current_user

        super(BasisModel, self).save(*args, **kwargs)

    def delete(self, *args, **kwargs):
        self.deleted = True
        self.save()

    def restore(self, *args, **kwargs):
        self.deleted = False
        self.save()



# -*- coding: utf8 -*-
import os
from setuptools import setup, find_packages

os.environ['DJANGO_SETTINGS_MODULE'] = 'tests.settings'

setup(
    name="django-basis",
    version='0.1.6',
    url='http://github.com/frecar/django-basis',
    author='Fredrik Nygård Carlsen',
    author_email='me@frecar.no',
    description='Simple reusable django app for basic model functionality',
    packages=find_packages(exclude='tests'),
    tests_require=[
        'django>=1.4',
    ],
    test_suite='runtests.runtests',
    include_package_data=True,
)
